% Solves Riemann problem for ideal gas

% r1, u1, p1  initial parameters for x<0
% r2, u2, p2  initial parameters for x>0
% rf, uf, pf    solution at the point (x,t)
% c1, c2        sound speeds for x<0 and for x>0
% s1, sr        velocties of the fronts of S(hock)W(ave)s
% shl, shr      velocties of the fronts of the heads of R(arefaction)Ws
% stl, str       velocties of the tails of RWs

function [rf,uf,pf]=riemann(x,t,r1,u1,p1,r2,u2,p2,gamma)

g(1)=0.5*(gamma-1.)/gamma; g(2)=0.5*(gamma+1.)/gamma; g(3)=2.*gamma/(gamma-1.);
g(4)=2./(gamma-1.); g(5)=2./(gamma+1.); g(6)=(gamma-1.)/(gamma+1.);
g(7)=0.5*(gamma-1.); g(8)=1./gamma; g(9)=gamma-1.;

s=x/t;
c1 = sqrt(p1/(r1*g(8)));
c2 = sqrt(p2/(r2*g(8)));
du=u2-u1;

% Vacuum solution
uvac=g(4)*(c1+c2)-du;
if uvac <= 0.
    disp(' Vacuum generated by given data');
    return;
end    

% Apply Newton method
% Initial guesses
pv=0.5*(p1+p2)-0.125*du*(r1+r2)*(c1+c2);
pmin=min(p1,p2); pmax=max(p1,p2); qrat=pmax/pmin;
if ( qrat <= 2. ) & ( pmin <= pv & pv <= pmax )
    p=max(1.0e-6,pv);
else
    if pv < pmin
        pnu=c1+c2-g(7)*du;
        pde=c1/(p1^g(1))+c2/(p2^g(1));
        p=(pnu/pde)^g(3);
    else
        gel=sqrt((g(5)/r1)/(g(6)*p1+max(1.0e-6,pv)));
        ger=sqrt((g(5)/r2)/(g(6)*p2+max(1.0e-6,pv)));
        p=(gel*p1+ger*p2-du)/(gel+ger); p=max(1.0e-6,p);
    end
end    
p0=p; err=1.;
while err > 1.0e-5
    if p <= p1
        prat=p/p1; fl=g(4)*c1*(prat^g(1)-1.); ffl=1./(r1*c1*prat^g(2));
    else
        a=g(5)/r1; b=g(6)*p1; qrt=sqrt(a/(b+p));
        fl=(p-p1)*qrt; ffl=(1.-0.5*(p-p1)/(b+p))*qrt;
    end
    if p <= p2
        prat=p/p2; fr=g(4)*c2*(prat^g(1)-1.); ffr=1./(r2*c2*prat^g(2));
    else
        a=g(5)/r2; b=g(6)*p2; qrt=sqrt(a/(b+p));
        fr=(p-p2)*qrt; ffr=(1.-0.5*(p-p2)/(b+p))*qrt;
    end
    p=p-(fl+fr+du)/(ffl+ffr);
    err=2.*abs(p-p0)/(p+p0); p0=p;
end
if p0 < 0.
    p0=1.0e-6;
end

% Velocity of CD 
u0=0.5*(u1+u2+fr-fl);

if s <= u0
    if p0 <= p1
        shl=u1-c1;
        if s <= shl
            rf=r1; uf=u1; pf=p1; return;
        else
            cml=c1*(p0/p1)^g(1); stl=u0-cml;
            if s > stl
                rf=r1*(p0/p1)^g(8); uf=u0; pf=p0; return;
            else
                uf=g(5)*(c1+g(7)*u1+s); c=g(5)*(c1+g(7)*(u1-s));
                rf=r1*(c/c1)^g(4); pf=p1*(c/c1)^g(3);
                return;
            end
        end
    else
        pml=p0/p1; sl=u1-c1*sqrt(g(2)*pml+g(1));
        if s <= sl
            rf=r1; uf=u1; pf=p1; return;
        else
            rf=r1*(pml+g(6))/(pml*g(6)+1.); uf=u0; pf=p0; return;
        end
    end
else
    if p0 > p2
        pmr=p0/p2; sr=u2+c2*sqrt(g(2)*pmr+g(1));
        if s >= sr
            rf=r2; uf=u2; pf=p2; return;
        else
            rf=r2*(pmr+g(6))/(pmr*g(6)+1.); uf=u0; pf=p0; return;
        end
    else
        shr=u2+c2;
        if s >= shr
            rf=r2; uf=u2; pf=p2; return;
        else
            cmr=c2*(p0/p2)^g(1); str=u0+cmr;
            if s <= str
                rf=r2*(p0/p2)^g(8); uf=u0; pf=p0; return;
            else
                uf=g(5)*(-c2+g(7)*u2+s); c=g(5)*(c2-g(7)*(u2-s));
                rf=r2*(c/c2)^g(4); pf=p2*(c/c2)^g(3);
                return;
            end
        end
    end
end
return;